# Ignore

Ignore is a lightweight node.js readable stream (** won't implemented until 0.2.x **) to apply minimatch ignore rules (like .gitignore). 

And it also has some utility methods to get filtered files and directories.

## Installation

	npm install ignore --save
	
## Usage

```js
var ignore = require('ignore');
```

### Filter the given paths

```js
var filtered = ignore({
	ignore: [
		'.abc/*',
		'!.abc/d/'
	]
}).filter([
	'.abc',
	'.abc/a.js',
	'.abc/d/e.js'
]);

console.log(filtered); // ['.abc/d/e.js'];
```

### Filter the cwd

```js
ignore({
	cwd: '/path/to/cwd/',
	ignoreFiles: [
		'.gitignore'
	],
	// include files each of which starts with a `'.'` 
	dot: true
}).filtered(function(err, files){
	console.log(files);
});
```

## Methods

### ignore(options)

##### options.ignoreFiles `Array`

The array of ignore files.

##### options.ignore `Array`

The array of ignore rules. 

Notice that the rules of `options.ignore` will be added before the `options.ignoreFiles`, so `options.ignore` has much lower priority.

##### options.dot `boolean=false`

Include .(dot) files in normal matches and globstar matches. Note that an explicit dot in a portion of the pattern will always match dot files.

##### options.`<xxxxx>` 

Other [`ignore-rules`](https://github.com/kaelzhang/ignore-rules) options.

### .filter(paths)

Filters the given array of pathnames, and returns the filtered array.

##### paths `Array.<String>`

The array of paths to be filtered

### .filtered(callback)

Get all filtered pathnames(files and directories) inside the `options.cwd`, according to the `options.ignoreFiles` and `options.ignore`.

##### callback `function(err, paths)`

##### paths `Array`

The array of filtered pathnames

### .addRuleFiles(files)

##### files `String|Array.<String>`

Add one or more ignore files

### .addRules(rules)

##### rules `String|Array.<String>`

Add one or more ignore rules
