'use strict';

module.exports = ignore;

var Readable    = require('stream').Readable;
var node_util   = require('util');
var node_fs     = require('fs');
var node_path   = require('path');
var glob        = require('glob');

var ignoreRules = require('ignore-rules');


function ignore (options) {
    return new Ignore(options);
}

ignore.Ignore = Ignore;

// Select the first existing file of the file list
ignore.select = function (files) {
    var exists;

    files.some(function (file) {
        if( node_fs.existsSync(file) ){
            exists = file;
            return true;
        }
    });

    return exists;
};


function Ignore (options) {
    options = options || {};

    Readable.call(this, options);

    this.cwd = options.cwd ?
        node_path.resolve(options.cwd) :
        process.cwd();
    
    this.options = options;

    options.ignore = options.ignore || [
        // Some files or directories which we should ignore for most cases.
        '.git',
        '.svn',
        '.DS_Store'
    ];

    this.ignoreRules = ignoreRules(options);
    
    this.ignoreFiles = [];
    this.addRuleFiles(options.ignoreFiles);
};

node_util.inherits(Ignore, Readable);


// @implementation
// stream.Readable interface
Ignore.prototype._read = function(size) {
    // body...
};


// @param {Array} files
Ignore.prototype.filter = function(files) {
    return this.ignoreRules.filter(files);
};


// get filtered files
Ignore.prototype.filtered = function(callback) {
    var self = this;

    this._filelist(function (err, files) {
        if(err){
            return callback(err);
        }

        callback(null, self.filter(files));
    });
};


Ignore.prototype._filelist = function(callback) {
    glob('**/*',  {
        cwd: this.cwd,

        // Includes files which start with '.'
        dot: this.options.dot,

        // Add a / character to directory matches
        // i.e. [
        //     'foo/',
        //     'foo/a.js'
        // ]
        // instead of [
        //     'foo',
        //     'foo/a.js'
        // ]
        mark: true

    }, callback);
};


Ignore.prototype.addRules = function (rules) {
    this.ignoreRules.add(rules);
    return this;
};


// @param {Array.<path>|path} a
Ignore.prototype.addRuleFiles = function(files) {
    makeArray(files).forEach(this._addRuleFile, this);
    return this;
};


Ignore.prototype._addRuleFile = function (file) {
    if(this._checkRuleFile(file)){
        this.ignoreFiles.push(file);

        // make sure `file` is an absolute path
        file = node_path.resolve(this.cwd, file);

        var content;

        try {
            content = node_fs.readFileSync(file);
        } catch(e) {
        }

        if(content){
            this.addRules( content.toString().split(/\r?\n/) );
        }
    }
};


Ignore.prototype._checkRuleFile = function(file) {
    return file !== '.' &&
           file !== '..' &&
           ! ~ this.ignoreFiles.indexOf(file);
};


function makeArray (subject) {
    return Array.isArray(subject) ? 
        subject :
        subject === undefined || subject === null ?
            [] :
            [subject];
}

