'use strict';

var ignore = require('../');
var node_path = require('path');
var expect = require('chai').expect;

describe("could apply ignoreRules", function(){
    it("could has exceptions of ignore rules ", function(){
        var filtered = ignore({
            ignore: [
                '.abc/*',
                '!.abc/d/'
            ]

        }).filter([
            '.abc/abc.js',
            '.abc/d/e.js'
        ]);

        expect(filtered).to.deep.equal(['.abc/d/e.js']);
    });

    it("could read rules from the ignore file", function(){
        var filtered = ignore({
            cwd: node_path.join(__dirname, 'fixtures'),
            ignoreFiles: [
                '.ignore'
            ],
            // include files each of which starts with a `'.'` 
            dot: true

        }).filter([
            '.abc/abc.js',
            '.abc/d/e.js',
            '.blah'
        ]);

        expect(filtered).to.deep.equal(['.abc/d/e.js']);
    });

    it("will ignore comments", function(){
        var ig = ignore({
            cwd: node_path.join(__dirname, 'fixtures'),
            ignoreFiles: [
                '.ignore2'
            ],

            ignore: [],

            // include files each of which starts with a `'.'` 
            dot: true

        });

        var files = [
            '.abc/abc.js',
            '.abc/d/e.js',
            '.blah'
        ];

        var filtered = ig.filter(files);

        expect(filtered).to.deep.equal(['.abc/d/e.js', '.blah']);
    });
});

describe("could filtered the cwd with the specified ignore files", function(){
    it("could read files list from the cwd", function(done){
        ignore({
            cwd: node_path.join(__dirname, 'fixtures'),
            ignoreFiles: [
                '.ignore3'
            ],
            // include files each of which starts with a `'.'` 
            dot: true

        }).filtered(function (err, filtered) {
            expect(err).to.equal(null);

            done();
        });
    });
});

